#ifndef CGAL_BERNSTEIN_BASIS_FO_H
#define CGAL_BERNSTEIN_BASIS_FO_H

#include "Types.h"
#include "Curve_constants.h"
#include "Helpers.h"


// Template function object class which implements the Bernstein polynomial
// representation of a Bezier curve and can be used for calculating points
// on this Bezier curve at given parameter value t
// Parameters:
//    NumType_    the number type of the points coordinates
template <class NumType_> 
class Bernstein_basis_FO {
public:
    typedef NumType_ NumType;
    typedef CGAL::Cartesian<NumType> REP;
    typedef CGAL::Point_2<REP> Point;   
    typedef vector<Point> Points;
    typedef CGAL::Vector_2<REP> Vector;
    typedef vector<Vector> Vectors;

    Bernstein_basis_FO() : contr_pts(NULL), a(LOW_BOUND), b(UP_BOUND) {
    }
    
    Bernstein_basis_FO(Points * contr_points, NumType a_=LOW_BOUND, 
		       NumType b_=UP_BOUND) : 
	contr_pts(contr_points), a(a_), b(b_) {
    }    

    // operator(): NumType -> Point
    // Parameters:
    //   t    the parameter value
    // Pre: a<=t<=b
    //      there is at least one control point
    // Returns: a point on the Bezier curve calculated with the Berstein 
    //          basis formula at t
    Point operator() (NumType t) const {	     	
	assert(!(*contr_pts).empty());
	assert(a<=t && t<=b);
	int n = (*contr_pts).size()-1;	
	if(t==a) return (* contr_pts)[0];	
	if(t==b) return (* contr_pts)[n];
        
        NumType c[n + 1];
	binomials(c, n);
	NumType t_i;
	Vector v(CGAL::NULL_VECTOR);
	for(int i=0; i<=n; i++) {
	    t_i = c[i]*power(t,i)*power((1-t),(n-i));
	    v = v + ((*contr_pts)[i] - CGAL::ORIGIN)*t_i;
	}
	return (CGAL::ORIGIN + v);	
    }
    
private:
    Points * contr_pts;
    NumType a, b;
};
#endif //CGAL_BERNSTEIN_BASIS_FO_H
